<?php

namespace Drupal\site_settings;

use Drupal\Core\Entity\Query\QueryFactory;
use Drupal\Core\Entity\EntityManager;

/**
 * Class SiteSettingsLoader.
 *
 * @package Drupal\site_settings
 */
class SiteSettingsLoader {

  /**
   * Drupal\Core\Entity\Query\QueryFactory definition.
   *
   * @var Drupal\Core\Entity\Query\QueryFactory
   */
  protected $entityQuery;

  /**
   * Drupal\Core\Entity\EntityManager definition.
   *
   * @var Drupal\Core\Entity\EntityManager
   */
  protected $entityManager;

  /**
   * Variable to store the loaded settings.
   *
   * @var array
   */
  protected $settings;

  /**
   * Constructor.
   */
  public function __construct(QueryFactory $entity_query, EntityManager $entity_manager) {
    $this->entityQuery = $entity_query;
    $this->entityManager = $entity_manager;
  }

  /**
   * Load site settings by fieldset.
   *
   * @param string $fieldset
   *   The name of the fieldset.
   *
   * @return array
   *   All settings within the given fieldset.
   */
  public function loadByFieldset($fieldset) {
    $this->loadAll();
    $fieldset = $this->fieldsetKey($fieldset);
    return $this->settings[$fieldset];
  }

  /**
   * Load site settings by fieldset.
   *
   * @param bool $rebuild_cache;
   *   Force rebuilding of the cache by setting to true.
   *
   * @return array
   *   All settings.
   */
  public function loadAll($rebuild_cache = FALSE) {
    if (!$rebuild_cache && $cache = \Drupal::cache()->get(SITE_SETTINGS_CACHE_CID)) {
      $this->settings = $cache->data;
    }
    else {
      $this->rebuildCache();
    }
    return $this->settings;
  }

  /**
   * Rebuild the cache.
   */
  public function rebuildCache() {
    $this->buildSettings();
    \Drupal::cache()->set(SITE_SETTINGS_CACHE_CID, $this->settings);
  }

  /**
   * Build the settings array.
   */
  private function buildSettings() {
    $query = $this->entityQuery->get('site_setting_entity');
    if ($ids = $query->execute()) {

      // Get the settings.
      $setting_entities = $this->entityManager
        ->getStorage('site_setting_entity')
        ->loadMultiple($ids);

      // Get entity type configurations at once for performance.
      $entities = [];
      $entity_type = \Drupal::entityTypeManager()->getStorage('site_setting_entity_type');
      if ($entity_type) {
        $entities = $entity_type->getEntities();
      }

      foreach ($setting_entities as $entity) {

        // Get data.
        $fieldset = $entity->fieldset->getValue()[0]['value'];
        $fieldset = $this->fieldsetKey($fieldset);
        $type = $entity->type->getValue()[0]['target_id'];
        $multiple = (isset($entities[$type]) ? $entities[$type]->multiple : FALSE);

        // If allowing multiple, set as array of entities.
        if ($multiple) {
          if (!isset($this->settings[$fieldset][$type])) {
            $this->settings[$fieldset][$type] = [];
          }

          $this->settings[$fieldset][$type][] = $this->getValues($entity);
        }
        else {
          $this->settings[$fieldset][$type] = $this->getValues($entity);
        }
      }

      // Get all possibilities and fill with empty values.
      $bundles = \Drupal::entityManager()
        ->getStorage('site_setting_entity_type')
        ->loadMultiple();
      foreach ($bundles as $bundle) {
        $fieldset = $this->fieldsetKey($bundle->fieldset);
        $id = $bundle->id();

        // Only fill if not yet set.
        if (!isset($this->settings[$fieldset][$id])) {
          $this->settings[$fieldset][$id] = '';
        }
      }
    }
  }

  /**
   * Get the values from the entity and return in as simple an array possible.
   *
   * @param object $entity
   *   Field Entity.
   *
   * @return mixed
   *   The values.
   */
  private function getValues($entity) {
    $build = [];
    $fields = $entity->getFields();
    foreach ($fields as $key => $field) {
      $field_definition = $field->getFieldDefinition();

      // Exclude fields on the object that are base config.
      if (!method_exists(get_class($field_definition), 'isBaseField') || !$field_definition->isBaseField()) {
        if ($value = $field->getValue()) {

          // Store the values in as flat a way as possible based on what is set.
          if (count($value) <= 1) {
            $item = reset($value);
            if (count(reset($value)) <= 1) {
              $build[$key] = reset($item);
            }
            else {
              $build[$key] = $item;
            }
          }
          else {
            $build[$key] = $value;
          }

        }
      }
    }
    return count($build) > 1 ? $build : reset($build);
  }

  /**
   * Create a lowercase key with no spaces from the fieldset label.
   *
   * @param string $fieldset
   *   The fieldset key.
   */
  private function fieldsetKey($fieldset) {
    return strtolower(str_replace(' ', '_', $fieldset));
  }

}
